declare name        "Ambisonic Encoder";
declare version     "1.0";
declare author      "Pierre Lecomte";
declare license     "CC-BY-NC-SA-4.0";
declare copyright   "(c) Pierre Lecomte";
declare options "[osc:on]";

// Changelog
// (2021-04-24)
// - Revert to azimuth-elevation spherical coordinate system $(\theta, \phi)$.

//###Encoder###
// This tool encodes $S$ sources as point sources in an Ambisonic sound scene up to a maximal degree $L$.
//
// ## Point source
// The $i$-th source, with $i \in \\{1, \cdots, S\\}$ carries a signal denoted $s(z)$ in the discrete domain. Encoded as a point source, its position is $(r_s, \theta_s, \phi_s)$ from origin and it emits a spherical wave.
// The near field filters are included to encode the radial distance $r_s$ information. (see [radial.lib]({% link docs/radial.md %}#near-field-filters)). 
// In the current implementation, these filters are stabilized with near field compensation filter at radius $r_\text{spk}=1$ m (see [radial.lib]({% link docs/radial.md %}#stabilization-of-nf-filters-with-nfc-filters)).
// The resulting Ambisonic components are given by:
//
// $$\begin{equation}
// b_{l,m}(z) = s(z) \frac{F_l(r_s, z)}{F_l(1, z)}  Y_{l,m}(\theta_s, \phi_s)
// \label{eq:point_source}
// \end{equation}$$
//
// Note that to avoid exessing gain for small radius $r_s < 1$ m, the minimum $r_s$ radius is limited at $r_s = 0.75$ m.
// {:.info}
//
// ### Plane wave case
// If the source radius $r_s$ is set to $r_\text{spk} = 1$ m, the source is encoded as a plane wave with direction $(\theta_s, \phi_s)$. In fact the radial term in Eq. \eqref{eq:point_source} is equal to unity.
// The Ambisonic components become:
//
// $$\begin{equation}
// b_{l,m}(z) = s(z) Y_{l,m}(\theta_s, \phi_s)
// \end{equation}$$
//
// ## Compilation parameters
// - `L`: maximal Spherical Harmonics degree (i.e., Ambisonics order), $L > 0$,
// - `S`: number of source to encode, $S > 0$.
//
// ## Inputs / Outputs
// - Inputs: $S$
// - Outputs: $(L+1)^2$
//
// ## User Interface
// For the $i$-th source:
//
// |             Element         |     OSC        | Min value | Max value | Step |
// |:---------------------------:|:--------------:|:---------:|:---------:|:----:|
// |          Gain (dB)          |   `gain_i`     |    -20    |    20     |  0.1 |
// |       Radius $r$) (m)       |  `radius_i`    |    0.75   |    50     | 0.01 |
// | Azimuth $\theta$ ($^\circ$) | `azimuth_i`    |    -180   |    180    | 0.01 |
// | Elevation $\phi$ ($^\circ$) |  `elevation_i` |    -90    |    90     | 0.01 |
//
//
//######

import("stdfaust.lib");
import("ylm.lib");
import("radial.lib");

// COMPILATION PARAMETERS
L = 3; // maximal SH degree
S = 2; // number of inputs

// DO NOT EDIT BELOW HERE
rspk = 1; // speaker radius (for near-field filters stabilization)

// User interface
g(i)	=	hslider("[%i+1][unit:dB][osc:/gain_%i -20 20][style:knob]Gain %2i",0,-20,20,0.1): ba.db2linear : si.smoo; // gain
r(i)	=	hslider("[%i+2][unit:m][osc:/radius_%i 0.5 50][style:knob]Radius %2i", 1, 0.75, 50, 0.01) : si.smoo;// radius
t(i)	=	hslider("[%i+3][unit:°][osc:/azimuth_%i -180 180][style:knob]Azimuth %2i", 0, -180, 180, 0.01)*ma.PI/180 : si.smoo; // azimuth
p(i)	=	hslider("[%i+4][unit:°][osc:/elevation_%i -90 90][style:knob]Elevation %2i", 0, -90, 90, 0.01)*ma.PI/180 : si.smoo; // elevation

source(i) =     hgroup("Source %2i",_*g(i)<:par(l, L+1, nf(l,r(i),rspk)<:par(i,2*l+1,_)):>yvec((L+1)^2, t(i), p(i)));

process = par(i, S, source(i)):>si.bus((L+1)^2);
